/*
HMValueTransformer.m

Author: Makoto Kinoshita

Copyright 2004-2006 The Shiira Project. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted 
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions 
  and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of 
  conditions and the following disclaimer in the documentation and/or other materials provided 
  with the distribution.

THIS SOFTWARE IS PROVIDED BY THE SHIIRA PROJECT ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE SHIIRA PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/

#import <Foundation/Foundation.h>

#import "HMValueTransformer.h"

@implementation HMDateValueTransformer

//--------------------------------------------------------------//
#pragma mark -- Initialize --
//--------------------------------------------------------------//

+ (void)load
{
    NSAutoreleasePool*  pool;
    pool = [[NSAutoreleasePool alloc] init];
    
    HMDateValueTransformer* transformer;
    transformer = [[HMDateValueTransformer alloc] init];
    
    [self setValueTransformer:transformer forName:@"HMDateValueTransformer"];
    
    [pool release];
}

//--------------------------------------------------------------//
#pragma mark -- NSValueTransformer method --
//--------------------------------------------------------------//

+ (Class)transformedValueClass
{
    return [NSString class];
}

+ (BOOL)allowsReverseTransformation
{
    return NO;
}

- (id)transformedValue:(id)value
{
    // Check value type
    if (![value isKindOfClass:[NSDate class]]) {
        return @"--/--/--";
    }
    
    // Return formatted string
    return [value descriptionWithCalendarFormat:@"%y/%m/%d %H:%M" 
            timeZone:nil 
            locale:[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]];
}

@end

#pragma mark -

@implementation HMSetValueTransformer

//--------------------------------------------------------------//
#pragma mark -- Initialize --
//--------------------------------------------------------------//

+ (void)load
{
    NSAutoreleasePool*  pool;
    pool = [[NSAutoreleasePool alloc] init];
    
    HMSetValueTransformer*  transformer;
    transformer = [[HMSetValueTransformer alloc] init];
    
    [self setValueTransformer:transformer forName:@"HMSetValueTransformer"];
    
    [pool release];
}

//--------------------------------------------------------------//
#pragma mark -- NSValueTransformer method --
//--------------------------------------------------------------//

+ (Class)transformedValueClass
{
    return [NSString class];
}

+ (BOOL)allowsReverseTransformation
{
    return NO;
}

- (id)transformedValue:(id)value
{
    // Check value type
    if (![value isKindOfClass:[NSSet class]]) {
        return @"";
    }
    
    // Return formatted string
    NSMutableString*    string;
    NSArray*            array;
    string = [NSMutableString string];
    array = [value allObjects];
    
    int i;
    for (i = 0; i < [array count]; i++) {
        NSString*   str;
        str = [array objectAtIndex:i];
        if (i == [array count] - 1) {
            [string appendString:str];
        }
        else {
            [string appendFormat:NSLocalizedString(@"%@, ", nil), str];
        }
    }
    
    return string;
}

@end
